"use strict";
{
	const C3 = self.C3;

	C3.Behaviors.skymen_RadialProgress.Instance = class RadialProgressInstance extends C3.SDKBehaviorInstanceBase
	{
		constructor(behInst, properties)
		{
			super(behInst);
			
			//this._testProperty = 0;
			
			this.needsRedraw = true;
			
			if (properties)
			{
				[
				  this["resolution"], //int
				  this["circular"], //bool
				  this["innerRadiusX"], //float
				  this["innerRadiusY"], //float
				  this["outerRadiusX"], //float
				  this["outerRadiusY"], //float
				  this["value"], //float
				  this["maximum"], //float
				  this["inverted"] //bool
				] = properties;
			}

			// Opt-in to getting calls to Tick()
			this._StartTicking();
		}
		
		Release()
		{
			super.Release();
		}
		
		SaveToJson()
		{
			let keys = ["resolution", //int
				"circular", //bool
				"innerRadiusX", //float
				"innerRadiusY", //float
				"outerRadiusX", //float
				"outerRadiusY", //float
				"value", //float
				"maximum", //float
				"inverted" //bool
			]
			
			let ret = {}
			keys.forEach(key => {
				ret[key] = this[key];
			})
			return ret;
		}
		
		LoadFromJson(o)
		{
			Object.keys(o).forEach(key => {
				this[key] = o[key];
			})
		}

		
		Tick()
		{
			const dt = this._runtime.GetDt(this._inst);
			const time = this._runtime.GetGameTime()
			const wi = this._inst.GetWorldInfo();
			let bboxChanged = false;
			
			let maximum = this["maximum"];
			let progress = this["value"];
			let inverted = this["inverted"];
			let resolution = this["resolution"];
			let innerRadiusX = this["innerRadiusX"];
			let innerRadiusY = this["circular"]? innerRadiusX : this["innerRadiusY"];
			let outerRadiusX = this["outerRadiusX"];
			let outerRadiusY = this["circular"]? outerRadiusX : this["outerRadiusY"];
			
			//indeterminate
			if (maximum <= 0) {
				if (!this.lastAngle) this.lastAngle = wi.GetAngle();
				this.needsRedraw = true;
				wi.SetAngle(wi.GetAngle() + C3.toRadians(180 * dt))
				//progress = (Math.sin(5 * time) + 0.7)/2;
				progress = 0.1;
				maximum = 1;
				//inverted = (Math.sin(5 * time - 0.01) - Math.sin(5 *time + 0.01)) <= 0
				bboxChanged = true;
			} else if (this.lastAngle){
				wi.SetAngle(this.lastAngle);
				bboxChanged = true;
				this.lastAngle = null;
			}
			if (bboxChanged) {
				wi.SetBboxChanged()
			}
			
			if (!this.needsRedraw) return;
			
			const needsMeshUpdate = !wi.HasMesh() || (wi._meshInfo.sourceMesh._hsize !== resolution && resolution > 2)
			if (needsMeshUpdate) {
				wi.CreateMesh(resolution, 2);
			}
			
			innerRadiusX = Math.min(innerRadiusX, outerRadiusX);
			innerRadiusY = Math.min(innerRadiusY, outerRadiusY);
			progress = Math.min(Math.max(0, progress), maximum);
			
			const startAngle = C3.toDegrees(wi.GetAngle());
			const endAngle = 360 * progress * (inverted? -1 : 1) / maximum + startAngle;
			const step = (endAngle - startAngle) / (resolution - 1);
		
			const width = wi.GetWidth();
			const height = wi.GetHeight();
			
			for(let i = 0; i < resolution; i++) {
				let curAngle = C3.toRadians(startAngle + step * i);
				bboxChanged = wi.SetMeshPoint(i, 0, {
					x: 0.5 + outerRadiusX * Math.cos(curAngle) / width,
					y: 0.5 + outerRadiusY * Math.sin(curAngle) / height
				})
				if (bboxChanged) {
					wi.SetBboxChanged()
				}
				bboxChanged = wi.SetMeshPoint(i, 1, {
					x: 0.5 + innerRadiusX * Math.cos(curAngle) / width,
					y: 0.5 + innerRadiusY * Math.sin(curAngle) / height
				})
				if (bboxChanged) {
					wi.SetBboxChanged()
				}
			}
			
			if (bboxChanged) {
				wi.SetBboxChanged()
			}
			
			this.needsRedraw = false;
		}
		

		GetDebuggerProperties()
		{
			let props = this.SaveToJson()
			let propertiesArr = [];
			Object.keys(props).forEach(prop => {
				propertiesArr.push(
					{name: prop[0].toUpperCase() + prop.slice(1), value: this[prop], onedit: v => this.setValue(prop, v)}
				)
			})
			return [{
				title: "RadialProgress",
				properties: propertiesArr,
			}];
		}
		
		setValue(name, val) {
			if(this[name] === val) return;
			this.needsRedraw = true;
			this[name] = val;
		}

		// timeline support
		GetPropertyValueByIndex(index)
		{
			return 0;
		}

		SetPropertyValueByIndex(index, value)
		{
			//set property value here
		}
	};
}